#include <engine/music.h>

/*
 * play_XA:
 *	Sends a command to the CD subsystem on which
 *	channel of an XA file it should play.
 */
void
play_XA(p_xa, channel)
struct XAFile *p_xa;
int channel;
{
	/* in case we are switching to a different XA file, set is_playing to 0 */
	p_xa->is_playing = 0;

	/* set is_playing to 1 to current XA file */
	p_xa->is_playing = 1;

	/* change channel */
	p_xa->filter.chan = channel;
	CdControl(CdlSetfilter, (unsigned char *)&p_xa->filter, 0);

	/* read from beginning of XA file */
	CdControl(CdlReadS, (unsigned char *)&p_xa->fp.pos, 0);
}

/*
 * cdsetup_XA:
 *	Setup ADPCM multi-channel play.
 */
char
cdsetup_XA(p_xa, p_name, channel, file)
struct XAFile *p_xa;
char *p_name;
int channel, file;
{
	if(!CdSearchFile(&p_xa->fp, p_name))
	{
		return 0; /* failure */
	}

	/* should not be playing by default */
	p_xa->is_playing = 0;

	/* set the filter (which ADPCM channel is playing) */
	p_xa->filter.file = file;
	p_xa->filter.chan = channel;
	CdControl(CdlSetfilter, (unsigned char *)&p_xa->filter, 0);

	return 1; /* success */
}

/*
 * cdsetup_mode:
 *	Set CD-ROM mode.
 */
void cdsetup_mode(mode)
unsigned int mode;
{
	CdControl(CdlSetmode, (unsigned char *)(&mode), 0);
	VSync(3); /* prevent CD motor screw-up; see LibOver47.pdf pg.196 for more info */
}

/*
 * cbready:
 *	Runs every time there is data available in the data buffer.
 *	Samples return their data sectors when they have finished playing, so
 *	to detect the end of the sample, we need to check which video sector
 *	channel number is being returned within the data sector.
 *
 *	This callback function is registered via CdReadyCallback.
 *
 *	FIXME: this should be in cd.c, not here
 */
void
cbready(status, p_result)
int status;
unsigned char *p_result;
{
	int i;

	unsigned short		current_channel;
	unsigned short		ID;

	/* if an XA channel is returning their data sector */
	switch(status)
	{
		case CdlDataReady:
			CdGetSector((unsigned int *)data_buffer, 8);

			/* check the first eight words to see if it's a video sector */
			ID = *((unsigned short *)(data_buffer)); /* ID == 0x0160 (352) */

			/* video sector channel number format = 1CCCCC0000000001 */
			current_channel = *((unsigned short *)(data_buffer+2));
			current_channel = (current_channel>>10) & 0x1f;

			/* debug */
			//if(ID != 0)
			//{
			//	printf("current channel == 0x%hx\nID == 0x%hx\n", current_channel, ID);

			//	printf("~~\n");
			//	// %lx is unsigned long
			//	//printf("%lx\n", *(unsigned long *)(data_buffer));
			//	//printf("%lx\n", *(unsigned long *)(data_buffer+8));
			//	//printf("%lx\n", *(unsigned long *)(data_buffer+16));
			//	//printf("%lx\n", *(unsigned long *)(data_buffer+24));
			//	//printf("%lx\n", *(unsigned long *)(data_buffer+32));
			//	//printf("%lx\n", *(unsigned long *)(data_buffer+48));
			//	//printf("%lx\n", *(unsigned long *)(data_buffer+60));
			//	//printf("%lx\n", *(unsigned long *)(data_buffer+72));
			//	//printf("~~\n");
			//}

			/*
			 * if this is a video sector, and if this is the
			 * channel being played, replay XA from beginning
			 */
			if( (ID == 352) && (current_channel == music.filter.chan) )
			{
				//play_XA(&music, music.filter.chan);
			}
			//else if(*p_cbready_cnt < *p_cbready_nsector) /* data sector */
			//{
			//	*p_cbready_cnt += 1;
			//}
			//else
			//{
			//	*p_cbready_cnt = CBREADY_CNT_FINISH;
			//}
			break;
	}
}

